@@ -14,7 +14,8 @@
 #include "lldb/Core/ClangForward.h"
 #include "lldb/Core/ConstString.h"
 #include "lldb/Core/UserID.h"
-#include "lldb/Symbol/ClangASTType.h"
+#include "lldb/Symbol/CompilerDecl.h"
+#include "lldb/Symbol/CompilerType.h"
 #include "lldb/Symbol/Declaration.h"
 
 #include "llvm/ADT/APSInt.h"
@@ -23,6 +24,31 @@
 
 namespace lldb_private {
 
+//----------------------------------------------------------------------
+// CompilerContext allows an array of these items to be passed to
+// perform detailed lookups in SymbolVendor and SymbolFile functions.
+//----------------------------------------------------------------------
+struct CompilerContext
+{
+    CompilerContext (CompilerContextKind t, const ConstString &n) :
+        type(t),
+        name(n)
+    {
+    }
+
+    bool
+    operator == (const CompilerContext &rhs) const
+    {
+        return type == rhs.type && name == rhs.name;
+    }
+
+    void
+    Dump () const;
+
+    CompilerContextKind type;
+    ConstString name;
+};
+
 class SymbolFileType :
     public std::enable_shared_from_this<SymbolFileType>,
     public UserID
@@ -34,6 +60,9 @@ class SymbolFileType :
         {
         }
 
+        SymbolFileType (SymbolFile &symbol_file, const lldb::TypeSP &type_sp);
+
+
         ~SymbolFileType ()
         {
         }
@@ -92,8 +121,8 @@ public:
           lldb::user_id_t encoding_uid,
           EncodingDataType encoding_uid_type,
           const Declaration& decl,
-          const ClangASTType &clang_qual_type,
-          ResolveState clang_type_resolve_state);
+          const CompilerType &compiler_qual_type,
+          ResolveState compiler_type_resolve_state);
     
     // This makes an invalid type.  Used for functions that return a Type when they
     // get an error.
@@ -239,22 +268,19 @@ public:
 
     // Get the clang type, and resolve definitions for any 
     // class/struct/union/enum types completely.
-    ClangASTType
-    GetClangFullType ();
+    CompilerType
+    GetFullCompilerType ();
 
     // Get the clang type, and resolve definitions enough so that the type could
     // have layout performed. This allows ptrs and refs to class/struct/union/enum 
     // types remain forward declarations.
-    ClangASTType
-    GetClangLayoutType ();
+    CompilerType
+    GetLayoutCompilerType ();
 
     // Get the clang type and leave class/struct/union/enum types as forward
     // declarations if they haven't already been fully defined.
-    ClangASTType 
-    GetClangForwardType ();
-
-    ClangASTContext &
-    GetClangASTContext ();
+    CompilerType 
+    GetForwardCompilerType ();
 
     static int
     Compare(const Type &a, const Type &b);
@@ -275,12 +301,6 @@ public:
     uint32_t
     GetEncodingMask ();
     
-    ClangASTType
-    CreateClangTypedefType (Type *typedef_type, Type *base_type);
-
-    bool
-    IsRealObjCClass();
-    
     bool
     IsCompleteObjCClass()
     {
@@ -302,14 +322,14 @@ protected:
     EncodingDataType m_encoding_uid_type;
     uint64_t m_byte_size;
     Declaration m_decl;
-    ClangASTType m_clang_type;
+    CompilerType m_compiler_type;
     
     struct Flags {
 #ifdef __GNUC__
         // using unsigned type here to work around a very noisy gcc warning
-        unsigned        clang_type_resolve_state : 2;
+        unsigned        compiler_type_resolve_state : 2;
 #else
-        ResolveState    clang_type_resolve_state : 2;
+        ResolveState    compiler_type_resolve_state : 2;
 #endif
         bool            is_complete_objc_class   : 1;
     } m_flags;
@@ -318,7 +338,7 @@ protected:
     GetEncodingType ();
     
     bool 
-    ResolveClangType (ResolveState clang_type_resolve_state);
+    ResolveClangType (ResolveState compiler_type_resolve_state);
 };
 
 // these classes are used to back the SBType* objects
@@ -327,28 +347,28 @@ class TypePair
 {
 public:
     TypePair () :
-        clang_type(),
+        compiler_type(),
         type_sp()
     {
     }
 
-    TypePair (ClangASTType type) :
-        clang_type(type),
+    TypePair (CompilerType type) :
+        compiler_type(type),
         type_sp()
     {
     }
     
     TypePair (lldb::TypeSP type) :
-    clang_type(),
-    type_sp(type)
+        compiler_type(),
+        type_sp(type)
     {
-        clang_type = type_sp->GetClangForwardType();
+        compiler_type = type_sp->GetForwardCompilerType ();
     }
     
     bool
     IsValid () const
     {
-        return clang_type.IsValid() || (type_sp.get() != nullptr);
+        return compiler_type.IsValid() || (type_sp.get() != nullptr);
     }
     
     explicit operator bool () const
@@ -359,21 +379,21 @@ public:
     bool
     operator == (const TypePair& rhs) const
     {
-        return clang_type == rhs.clang_type &&
+        return compiler_type == rhs.compiler_type &&
         type_sp.get() == rhs.type_sp.get();
     }
     
     bool
     operator != (const TypePair& rhs) const
     {
-        return clang_type != rhs.clang_type ||
+        return compiler_type != rhs.compiler_type ||
         type_sp.get() != rhs.type_sp.get();
     }
     
     void
     Clear ()
     {
-        clang_type.Clear();
+        compiler_type.Clear();
         type_sp.reset();
     }
     
@@ -382,8 +402,8 @@ public:
     {
         if (type_sp)
             return type_sp->GetName();
-        if (clang_type)
-            return clang_type.GetTypeName();
+        if (compiler_type)
+            return compiler_type.GetTypeName();
         return ConstString ();
     }
     
@@ -391,24 +411,27 @@ public:
     GetDisplayTypeName () const
     {
         if (type_sp)
-            return type_sp->GetClangForwardType().GetDisplayTypeName();
-        if (clang_type)
-            return clang_type.GetDisplayTypeName();
+            return type_sp->GetForwardCompilerType ().GetDisplayTypeName();
+        if (compiler_type)
+            return compiler_type.GetDisplayTypeName();
         return ConstString();
     }
     
     void
-    SetType (ClangASTType type)
+    SetType (CompilerType type)
     {
         type_sp.reset();
-        clang_type = type;
+        compiler_type = type;
     }
     
     void
     SetType (lldb::TypeSP type)
     {
         type_sp = type;
-        clang_type = type_sp->GetClangForwardType();
+        if (type_sp)
+            compiler_type = type_sp->GetForwardCompilerType ();
+        else
+            compiler_type.Clear();
     }
     
     lldb::TypeSP
@@ -417,72 +440,76 @@ public:
         return type_sp;
     }
     
-    ClangASTType
-    GetClangASTType () const
+    CompilerType
+    GetCompilerType () const
     {
-        return clang_type;
+        return compiler_type;
     }
     
-    ClangASTType
+    CompilerType
     GetPointerType () const
     {
         if (type_sp)
-            return type_sp->GetClangLayoutType().GetPointerType();
-        return clang_type.GetPointerType();
+            return type_sp->GetForwardCompilerType().GetPointerType();
+        return compiler_type.GetPointerType();
     }
     
-    ClangASTType
+    CompilerType
     GetPointeeType () const
     {
         if (type_sp)
-            return type_sp->GetClangFullType().GetPointeeType();
-        return clang_type.GetPointeeType();
+            return type_sp->GetForwardCompilerType ().GetPointeeType();
+        return compiler_type.GetPointeeType();
     }
     
-    ClangASTType
+    CompilerType
     GetReferenceType () const
     {
         if (type_sp)
-            return type_sp->GetClangLayoutType().GetLValueReferenceType();
-        return clang_type.GetLValueReferenceType();
+            return type_sp->GetForwardCompilerType ().GetLValueReferenceType();
+        else
+            return compiler_type.GetLValueReferenceType();
     }
 
-    ClangASTType
+    CompilerType
     GetTypedefedType () const
     {
         if (type_sp)
-            return type_sp->GetClangFullType().GetTypedefedType();
-        return clang_type.GetTypedefedType();
+            return type_sp->GetForwardCompilerType ().GetTypedefedType();
+        else
+            return compiler_type.GetTypedefedType();
     }
 
-    ClangASTType
+    CompilerType
     GetDereferencedType () const
     {
         if (type_sp)
-            return type_sp->GetClangFullType().GetNonReferenceType();
-        return clang_type.GetNonReferenceType();
+            return type_sp->GetForwardCompilerType ().GetNonReferenceType();
+        else
+            return compiler_type.GetNonReferenceType();
     }
     
-    ClangASTType
+    CompilerType
     GetUnqualifiedType () const
     {
         if (type_sp)
-            return type_sp->GetClangLayoutType().GetFullyUnqualifiedType();
-        return clang_type.GetFullyUnqualifiedType();
+            return type_sp->GetForwardCompilerType ().GetFullyUnqualifiedType();
+        else
+            return compiler_type.GetFullyUnqualifiedType();
     }
     
-    ClangASTType
+    CompilerType
     GetCanonicalType () const
     {
         if (type_sp)
-            return type_sp->GetClangFullType().GetCanonicalType();
-        return clang_type.GetCanonicalType();
+            return type_sp->GetForwardCompilerType ().GetCanonicalType();
+        return compiler_type.GetCanonicalType();
     }
     
-    clang::ASTContext *
-    GetClangASTContext () const
+    TypeSystem *
+    GetTypeSystem () const
     {
-        return clang_type.GetASTContext();
+        return compiler_type.GetTypeSystem();
     }
     
     lldb::ModuleSP
@@ -493,10 +520,13 @@ public:
         return lldb::ModuleSP();
     }
 protected:
-    ClangASTType clang_type;
+    CompilerType compiler_type;
     lldb::TypeSP type_sp;
 };
     
+// the two classes here are used by the public API as a backend to
+// the SBType and SBTypeList classes
+    
 class TypeImpl
 {
 public:
@@ -509,28 +539,28 @@ public:
     
     TypeImpl (const lldb::TypeSP &type_sp);
     
-    TypeImpl (const ClangASTType &clang_type);
+    TypeImpl (const CompilerType &compiler_type);
     
-    TypeImpl (const lldb::TypeSP &type_sp, const ClangASTType &dynamic);
+    TypeImpl (const lldb::TypeSP &type_sp, const CompilerType &dynamic);
     
-    TypeImpl (const ClangASTType &clang_type, const ClangASTType &dynamic);
+    TypeImpl (const CompilerType &compiler_type, const CompilerType &dynamic);
     
-    TypeImpl (const TypePair &pair, const ClangASTType &dynamic);
+    TypeImpl (const TypePair &pair, const CompilerType &dynamic);
 
     void
     SetType (const lldb::TypeSP &type_sp);
     
     void
-    SetType (const ClangASTType &clang_type);
+    SetType (const CompilerType &compiler_type);
     
     void
-    SetType (const lldb::TypeSP &type_sp, const ClangASTType &dynamic);
+    SetType (const lldb::TypeSP &type_sp, const CompilerType &dynamic);
     
     void
-    SetType (const ClangASTType &clang_type, const ClangASTType &dynamic);
+    SetType (const CompilerType &compiler_type, const CompilerType &dynamic);
     
     void
-    SetType (const TypePair &pair, const ClangASTType &dynamic);
+    SetType (const TypePair &pair, const CompilerType &dynamic);
     
     TypeImpl&
     operator = (const TypeImpl& rhs);
@@ -575,11 +605,11 @@ public:
     TypeImpl
     GetCanonicalType() const;
     
-    ClangASTType
-    GetClangASTType (bool prefer_dynamic);
+    CompilerType
+    GetCompilerType (bool prefer_dynamic);
     
-    clang::ASTContext *
-    GetClangASTContext (bool prefer_dynamic);
+    TypeSystem *
+    GetTypeSystem (bool prefer_dynamic);
     
     bool
     GetDescription (lldb_private::Stream &strm, 
@@ -592,7 +622,7 @@ private:
 
     lldb::ModuleWP m_module_wp;
     TypePair m_static_type;
-    ClangASTType m_dynamic_type;
+    CompilerType m_dynamic_type;
 };
 
 class TypeListImpl
@@ -750,7 +780,7 @@ class TypeAndOrName
 public:
     TypeAndOrName ();
     TypeAndOrName (lldb::TypeSP &type_sp);
-    TypeAndOrName (const ClangASTType &clang_type);
+    TypeAndOrName (const CompilerType &compiler_type);
     TypeAndOrName (const char *type_str);
     TypeAndOrName (const TypeAndOrName &rhs);
     TypeAndOrName (ConstString &type_const_string);
@@ -772,10 +802,10 @@ public:
         return m_type_pair.GetTypeSP();
     }
     
-    ClangASTType
-    GetClangASTType () const
+    CompilerType
+    GetCompilerType () const
     {
-        return m_type_pair.GetClangASTType();
+        return m_type_pair.GetCompilerType();
     }
     
     void
@@ -788,7 +818,7 @@ public:
     SetTypeSP (lldb::TypeSP type_sp);
     
     void
-    SetClangASTType (ClangASTType clang_type);
+    SetCompilerType (CompilerType compiler_type);
     
     bool
     IsEmpty () const;
@@ -800,12 +830,12 @@ public:
     HasTypeSP () const;
     
     bool
-    HasClangASTType () const;
+    HasCompilerType () const;
     
     bool
     HasType () const
     {
-        return HasTypeSP() || HasClangASTType();
+        return HasTypeSP() || HasCompilerType();
     }
     
     void
@@ -825,60 +855,43 @@ class TypeMemberFunctionImpl
 {
 public:
     TypeMemberFunctionImpl() :
-        m_type(),
-        m_objc_method_decl(nullptr),
+        m_type (),
+        m_decl (),
         m_name(),
-        m_kind(lldb::eMemberFunctionKindUnknown)
+        m_kind (lldb::eMemberFunctionKindUnknown)
     {
     }
     
-    TypeMemberFunctionImpl (const ClangASTType& type,
+    TypeMemberFunctionImpl (const CompilerType& type,
+                            const CompilerDecl& decl,
                             const std::string& name,
                             const lldb::MemberFunctionKind& kind) :
-        m_type(type),
-        m_objc_method_decl(nullptr),
+        m_type (type),
+        m_decl (decl),
         m_name(name),
-        m_kind(kind)
+        m_kind (kind)
     {
     }
     
-    TypeMemberFunctionImpl (clang::ObjCMethodDecl *method,
-                            const std::string& name,
-                            const lldb::MemberFunctionKind& kind) :
-    m_type(),
-    m_objc_method_decl(method),
-    m_name(name),
-    m_kind(kind)
-    {
-    }
-    
-    TypeMemberFunctionImpl (const TypeMemberFunctionImpl& rhs) :
-        m_type(rhs.m_type),
-        m_objc_method_decl(rhs.m_objc_method_decl),
-        m_name(rhs.m_name),
-        m_kind(rhs.m_kind)
-    {
-    }
-    
-    TypeMemberFunctionImpl&
-    operator = (const TypeMemberFunctionImpl& rhs);
-    
     bool
     IsValid ();
     
     ConstString
     GetName () const;
-    
-    ClangASTType
+
+    ConstString
+    GetMangledName () const;
+
+    CompilerType
     GetType () const;
     
-    ClangASTType
+    CompilerType
     GetReturnType () const;
     
     size_t
     GetNumArguments () const;
     
-    ClangASTType
+    CompilerType
     GetArgumentAtIndex (size_t idx) const;
     
     lldb::MemberFunctionKind
@@ -892,8 +905,8 @@ protected:
     GetPrintableTypeName ();
 
 private:
-    ClangASTType m_type;
-    clang::ObjCMethodDecl *m_objc_method_decl;
+    CompilerType m_type;
+    CompilerDecl m_decl;
     ConstString m_name;
     lldb::MemberFunctionKind m_kind;
 };
@@ -909,8 +922,9 @@ public:
     {
     }
 
-    TypeEnumMemberImpl (const clang::EnumConstantDecl* enum_member_decl,
-                        const lldb_private::ClangASTType& integer_type);
+    TypeEnumMemberImpl (const lldb::TypeImplSP &integer_type_sp,
+                        const ConstString &name,
+                        const llvm::APSInt &value);
 
     TypeEnumMemberImpl (const TypeEnumMemberImpl& rhs) :
         m_integer_type_sp(rhs.m_integer_type_sp),
@@ -944,13 +958,13 @@ public:
     uint64_t
     GetValueAsUnsigned () const
     {
-        return *m_value.getRawData();
+        return m_value.getZExtValue();
     }
 
     int64_t
     GetValueAsSigned () const
     {
-        return (int64_t) *m_value.getRawData();
+        return m_value.getSExtValue();
     }
 
 protected:
